package com.blazinggames.retro.shared.search;

/**
* What good is a tank game if you are unable to shoot? This class holds
 * the round that is being fired and moves the round through 3D space for
 * the durration of it's life. 
 * ------------------------------------------------
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Billy D. Spelchan
 * @version 1.00.011016
 */
public class Projectile
{
    // *********************
    // ***** CONSTANTS *****
    // *********************

    public static final double GRAVITY = -9.8;


    // *********************
    // ***** VARIABLES *****
    // *********************

    protected double _x;
    protected double _y;
    protected double _height;

    protected double _xVelocity;
    protected double _yVelocity;
    protected double _heightVelocity;

    protected double _windX;
    protected double _windY;
    protected double _gravity;

    protected boolean _hasMoved;

    // ************************
    // ***** CONSTRUCTORs *****
    // ************************

    public Projectile(double x, double y, int dir, int ang, int power)
    {
        this(x, y, (double)dir, ang, power);
    }

    public Projectile(double x, double y, double dir, int ang, int power)
    {
        _x = x;
        _y = y;
        _height = 0;

        double rad = (double)ang * Math.PI / 180.0;
        _heightVelocity = Math.sin(rad) * (double)power;
        double remaining = (double)power - _heightVelocity;
        rad = (double)dir * Math.PI / 180.0;
        _xVelocity = Math.cos(rad) * remaining;
        _yVelocity = Math.sin(rad) * remaining;

        _windX = 0;
        _windY = 0;
        _gravity = GRAVITY;

        _hasMoved = false;
    }


    // *****************************************************
    // ***** METHODS - Information getting and setting *****
    // *****************************************************

    public double getX()
    {
        return _x;
    }

    public double getY()
    {
        return _y;
    }

    public double getHeight()
    {
        return _height;
    }

    public void setWind(int dir, int power)
    {
        double rad = (double)dir * Math.PI / 180.0;
        _windX = Math.cos(rad) * (double)power;
        _windY = Math.sin(rad) * (double)power;
    }

    public void setGravity(double g)
    {
        _gravity = g;
    }

    public boolean isAlive()
    {
        if ( (_hasMoved) && (_height <= 0) )
            return false;
        else
            return true;
    }

    // *****************************
    // ***** METHODS - ACTIONS *****
    // *****************************

    public boolean tick(int ms)
    {
        double time = ((double)ms / 1000.0);
        // first see if has hit ground
        double oldHeight = _height;
        _height += (_heightVelocity * time);
        if (_height <= 0)
        {
            double temp = oldHeight / Math.abs(_heightVelocity * time);
            time *= temp;
            _height = 0;
        }
        _heightVelocity += (_gravity * time);
        _x += (_xVelocity * time);
        _y += (_yVelocity * time);
        _xVelocity += (_windX * time);
        _yVelocity += (_windY * time);
        _hasMoved = true;
        if (_height <= 0)
            return false;
        else
            return true;
    }
}
